!- - - - -- - -- - -- - -- - - -- - --  -- - -- - -- - - -- - - - -- - --
! gfsnemsio2grb: 2018 Public Domain Jun Wang, Wesley Ebisuzaki
!
! This program converts a GFS Gaussian Grid nemsio file into grib2
!	the program handles most of the atm and sfc files
!
! This program is based on mkgfsnemsioctl.f  (5/12/2015 Jun Wang)
!
! The conversion to grib is done using wgrib2api
!   http://www.cpc.ncep.noaa.gov/products/wesley/wgrib2/wgrib2api.html
!
! The center, subcenter and process ids are encoded into the template file, template_512.grb
!  To change the center, subcenter, process ids, etc, you modify the template
!
! Requirements:  
!       New Fortran: newer f90 or f95, intel circa 2012 is ok
!       wgrib2 library
!
!   v1.0 April 20, 2018   Jun Wang, Wesley Ebisuzaki
!   v1.1 July 26, 2018    Wesley Ebisuzaki, more fields converted to grib, timing added
!   v1.2 Aug 21, 2018     Wesley Ebisuzaki, better timing information in grib files
!   v1.3 Aug 23, 2018     Wesley Ebisuzaki, added my_option for climate reanalysis
!                                           fix problem in precip, enkf analyses are started from
!                                           fhr=-3, and the bucket is emptied at fhr=0
!   v1.4 Sep 5, 2018      Wesley Ebisuzaki, climate reanalysis: add wind speed at 10m
!   v1.4.1 Sep 21, 2018   Wesley Ebisuzaki, compatible with gfortran version
!   v1.5                  Wesley Ebisuzaki, use hybrid pressure level
!                                           calc. P and Z for hybrid levels (sfg/atm)
!   v1.6 Nov 20, 2018     Wesley Ebisuzaki, turn off generation of PRES of model levels
!   v1.7 Dec 29, 2018     Wesley Ebisuzaki, close template, add fast and  prate_adjust
!                             fast: no ctl file, packing=aec instead of c2,
!                                        less text 
!   v1.8 jan 13, 2019     Wesley Ebisuzaki, fix prate_adjust, extra_var
!   v1.9 mar, 2019        Wesley Ebisuzaki, look for land mask, apply mask to certain fields
!                                           grib: use undefined instead of special value
!                                            landmask=  1 land
!                                                       0 open water
!                                                       2 ice covered water
!                                            look for template in template_512.grb,  $TEMPLATE,
!                                            and $HOME/bin_data/template_512.grb
!					     added CDCA (cloud amount)
!   v2.0 April, 2019      Wesley Ebisuzaki, add A option, use aec compression
!                                           add S option for simple packing
!                                           public release
!   v2.1 Sep 25, 2019     Wesley Ebisuzaki, apply land mask to more fields, TRANS, FLDCP, EVBS, 
!                                           EVCW, WATR, PEVPR, SLTYP, SSRUN
!                                           saved more data in the CORe nemsio file that mask
!					    option a: TSOIL instead of TMP
!   v2.2 Dec 20, 2019     Wesley Ebisuzaki, option p: hydrostatic surface pressure
!					    delz -> THICKNESS (old: hgt)
!					    dpres -> nothing (no grib equivalent, old PRES)
!   v2.3 Feb 20, 2020     Wesley Ebisuzaki, add -o (output file), work with wgrib2 v2.0.9 if template is U/V
!   V2.4 March 13, 2020   Wesley Ebisuzaki, do not write DPRES,PRATEB,CPRATB,TI, TG3,F10M, FFHH, FFMM, F10
!   v2.5 April 27, 2020   Wesley Ebisuzaki, SNOHF and Clear Sky Fluxes are time aveages add 
!                                           option F to fix, also "a" fixes it too
! - - - - -- - -- - -- - -- - - -- - --  -- - -- - -- - - -- - - - -- - --
!
  use nemsio_module
  use wgrib2api
  implicit none
!
!---------------------------------------------------------------------------
  character (len=30) :: itoa

  type(nemsio_gfile) :: gfile
  character(255) cin,cindx,cgrb,my_option, line
  real, allocatable :: data(:,:), data1(:), u10m(:), press0(:), press(:,:)
  real, allocatable :: land(:), press1(:)
  integer :: has_press0, has_press, n_ensemble, type_ensemble
  character(100) :: glev, ftime, refdate, fcst, ave, min,max,acc, xdef, ydef, precip_ave
  character(100) :: to_upper
  character(200) :: meta
  character(70) :: version = 'gfsnemsio2grb v2.5 4/2020   Wesley Ebisuzaki, Jun Wang'
  character(120) :: template
  character(2) :: packing
  real :: p_hybrid64

!---------------------------------------------------------------------------
!--- nemsio meta data
  integer nrec,im,jm,lm,idate(7),nfhour,tlmeta,nsoil,fieldsize
  real,allocatable       :: slat(:),dx(:)
  real(8),allocatable    :: lat(:)
  character(16),allocatable:: recname(:),reclevtyp(:)
  integer,allocatable:: reclev(:)
!---------------------------------------------------------------------------
!--- local vars
  character(3) cmon
  character(16) reclevtyp_sht
  character(32) ctldate,varname
  character(35) sweep_blanks
  real(8) dxctl,radi
  integer i,n,j,krec,iret,io_unit,idrt,data_unit
  logical u10m_avail, fast, prate_adjust, extra_var, has_landmask, land_var 
  logical :: fix_ave
  logical convert_tsoil, testmode
  integer :: strlen, fhzero, sfhour
  real, parameter :: undefined = 9.99e20, undefined_wgrib2 = 9.999e20
  real :: precip_factor
!---------------------------------------------------------------------------
!---------------------------------------------------------------------------
!
  has_press0 = 0
  has_press = 0
  n_ensemble = 0
  packing = 'c3'
  testmode = .false.
  prate_adjust = .false.
  fix_ave = .false.
  extra_var = .false.
  convert_tsoil = .false.

  write(*,*) 'version=', version

  call nemsio_init(iret=iret)
  if (iret/=0) then
     print *,'ERROR: nemsio_init '
     stop 1
  endif

! read options

  call get_command_argument(2,my_option)
  fast = (index(my_option,'f') .ne. 0)

  if (index(my_option,'a').ne.0) then
     prate_adjust = .true.
     fix_ave = .true.
     extra_var = .true.
     convert_tsoil = .true.
  endif

  testmode = (index(my_option,'p').ne.0)
  
  if (index(my_option,'A').ne.0) then
     packing = 'a'
  endif
  if (index(my_option,'S').ne.0) then
     packing = 's'
  endif
  if (index(my_option,'F').ne.0) then
     fix_ave = .true.
  endif

!  if (fast) packing='a'
! parse ensemble option:  eN  N = number of ensemble members
  i = index(my_option,'e')
  if (i.ne.0) then
     i = i + 1
     j = iachar(my_option(i:i)) - iachar('0')
     do while (j >= 0 .and. j <= 9)
         n_ensemble = 10*n_ensemble + j
         i = i + 1
         j = iachar(my_option(i:i)) - iachar('0')
     enddo
     write(*,*) 'n_ensemble=',n_ensemble
  endif

! get nemsio file name for reading
  call get_command_argument(1,cin)
  print *,'nemsio file=',trim(cin)
  if (trim(cin)=='') then
    print *,'usage: gfsnemsio2grb input_nemsio_file_name (option) (-o gribname)'
    print *,' documentation: https://www.cpc.ncep.noaa.gov/products/wesley/gfsnemsio2grb.html'
    stop 2
  endif
  cindx=trim(cin)//'.ctl'		! grads ctl file
  cgrb=trim(cin)//'.grb'		! output grib file

! read additional options
  n = command_argument_count()
  write(*,*) 'n=', n
  i = 3
  if (n >= i+1) then
     call get_command_argument(i,line)
     if (line == '-o') then
         call get_command_argument(i+1,cgrb)
         write(*,*) 'output grib file=',trim(cgrb)
         i=i+2
     endif
  endif

!---------------------------------------------------------------------------
!***  read nemsio grd header info
!---------------------------------------------------------------------------
!

! is it an control, ensmean memXXX
! type_ensemble:
!                = 0 (no ensemble info)
!                > 0 ensemble member
!                =-1 control
!                =-2 ensemble mean

  type_ensemble = 0
  if (n_ensemble > 0) then
     if (index(cin,'control').ne.0) type_ensemble = -1
     if (index(cin,'ensmean').ne.0) type_ensemble = -2
     i = index(cin,'mem')
     if (i.ne.0) then
         type_ensemble = 0
         i = i + 3
         j = iachar(cin(i:i)) - iachar('0')
         do while (j >= 0 .and. j <= 9)
             type_ensemble = 10*type_ensemble + j
             i = i + 1
             j = iachar(cin(i:i)) - iachar('0')
         enddo
     endif
     write(*,*) 'type of ensemble=',type_ensemble
  endif

  call nemsio_open(gfile,trim(cin),'READ',iret=iret)
  if (iret/=0) then
    print *,'Error: open nemsio file,',trim(cin),' iret=',iret
    stop 3
  endif

  call nemsio_getfilehead(gfile,iret=iret,nrec=nrec,dimx=im,dimy=jm, &
    dimz=lm,idate=idate,nfhour=nfhour,                               &
    nsoil=nsoil,tlmeta=tlmeta)

! 
   call nemsio_getheadvar(gfile,'fhzero',fhzero,iret)
   if (iret.ne.0) then
      write(*,*) ' fhzero not found, 0 used'
      fhzero = 0.0
   endif
   if (.not.fast) write(*,*) 'fhzero:', fhzero,' nfhour:',nfhour

   fieldsize=im*jm
   u10m_avail = .false.
   allocate(recname(nrec),reclevtyp(nrec),reclev(nrec))
   allocate(lat(fieldsize),slat(jm),dx(fieldsize))
   call nemsio_getfilehead(gfile,iret=iret,recname=recname,          &
       reclevtyp=reclevtyp,reclev=reclev,idrt=idrt) 

!  check for Gaussian grid
   if (idrt /= 4) then
       write(*,*) 'Error: expected idrt=4 (Gaussian grid), not ',idrt
       stop 4
   endif 
!  get Gaussian grid
   call splat4(idrt,jm,slat)
   radi=180.0/(4.*atan(1.0))
!omp parallel do private(j)
   do  j=1,jm
       lat(j) = asin(slat(j)) * radi
   enddo
   dxctl=360./(im*1.)

!   create template with Gaussian grid using wgrib2 -new_grid
    write(xdef,'(a,a,a,f10.6)') '0:', trim(itoa(im)),':',360.0/im
    write(ydef,'(f12.6,a,a)') lat(1),':',trim(itoa(jm))
    if (.not.fast) write(*,*) trim(xdef)
    if (.not.fast) write(*,*) 'ydef=',trim(ydef)

!   change grid

!    read template from . then $TEMPLATE and $HOME/bin_data

    template = 'template_512.grb'
    i = wgrib2a(template,'-set_var', 'TMP', '-new_grid_winds','earth','-new_grid','gaussian',xdef,ydef,'@mem:1')

    if (i /= 0) then
!       try finding data in $TEMPLATE
        call get_environment_variable('TEMPLATE',template,i)
        if (i /= 0) then
            template=trim(template) // '/bin_data/template_512.grb'
            i = wgrib2a(template,'-new_grid_winds','earth','-new_grid','gaussian',xdef,ydef,'@mem:1')
        endif
    endif
    if (i /= 0) then
!       try finding data in $HOME/bin_data/
        call get_environment_variable('HOME',template,i)
        if (i /= 0) then
            template=trim(template) // '/bin_data/template_512.grb'
            i = wgrib2a(template,'-new_grid_winds','earth','-new_grid','gaussian',xdef,ydef,'@mem:1')
        endif
    endif
    if (i /= 0) then
        write(*,*) 'problem reading template_512.grb'
        stop 5
    endif
    i = grb2_free_file(template)
    if (i /= 0) then
        write(*,*) 'problems freeing file ', trim(template)
        stop 6
    endif

!   make template with ensemble info

    if (type_ensemble == 0) then
!       ordinary, no ensemble info
        i = wgrib2a('@mem:1','-rewind_init','@mem:1','-grib','@mem:0')
    else if (type_ensemble == -1) then
!      control
       i = wgrib2a('@mem:1','-rewind_init','@mem:1','-set_ens_num','0','0',itoa(n_ensemble),&
            '-grib', '@mem:0')
       if (i /= 0) stop 7
    elseif (type_ensemble > 0) then
!      ensemble member #type_ensemble
       i = wgrib2a('@mem:1','-rewind_init','@mem:1','-set_ens_num','3',itoa(type_ensemble), &
           itoa(n_ensemble), '-grib','@mem:0')
    elseif (type_ensemble == -2) then
!      ensemble mean
       i = wgrib2a('@mem:1','-rewind_init','@mem:1','-set_ensm_derived_fcst','0', &
             itoa(n_ensemble), '-grib','@mem:0')
    else
      write(*,*) ' ensemble programming error, type_ensemble=',type_ensemble
      stop 8
    endif

!   find the start of the averaging period
!   nfhour = end of forecast in hours
!   fhzero = how often bucket is emptied
    if (nfhour <= 0 .or. fhzero .eq. 0) then
        sfhour = nfhour
    else
        sfhour = int( (nfhour-1)/fhzero )*fhzero
    endif

    write(refdate,'(i4.4,5i2.2)') idate(1:6)
    if (.not.fast) write(*,*) 'date=',idate(1:4), ' fhours=',sfhour,nfhour, trim(refdate)
    allocate (data(im,jm),data1(im*jm), u10m(im*jm))
    if (testmode) then
        allocate (press1(im*jm))
        press1 = 0.0
    endif

!   formats for ftime

    fcst= trim(itoa(nfhour)) // ' hour fcst'
    ave = trim(itoa(sfhour)) // '-' // trim(itoa(nfhour)) // ' hour ave fcst'
    if (prate_adjust) then
       precip_ave = '3-' // trim(itoa(nfhour)) // ' hour ave fcst'
       precip_factor=real(nfhour) / real(nfhour-3.0)
    else
       precip_ave = ave
       precip_factor=1.0
    endif

    max = trim(itoa(sfhour)) // '-' // trim(itoa(nfhour)) // ' hour max fcst'
    min = trim(itoa(sfhour)) // '-' // trim(itoa(nfhour)) // ' hour min fcst'
    acc = trim(itoa(sfhour)) // '-' // trim(itoa(nfhour)) // ' hour acc fcst'

!   look for land field, needed for mask

    do i = 1, nrec
	if (recname(i).eq.'land') then
            exit
        endif
    enddo

    if (i.eq.nrec+1) then
        has_landmask = .false.
    else
        allocate (land(im*jm))
	call nemsio_readrec(gfile,i,land,iret=iret)
	if (iret.ne.0) then
	    write(*,*) 'nemsio_readrec land problem return = ', iret
            stop 10 
        endif
        has_landmask = .true.
    endif
    if (.not.fast) write(*,*) 'has_landmask=',has_landmask

!   loop over all records
    do i = 1, nrec
	call nemsio_readrec(gfile,i,data1,iret=iret)
	if (iret.ne.0) then
	    write(*,*) 'nemsio_readrec problem return = ', iret
            stop 3
	endif

	varname = recname(i)
	glev = reclevtyp(i)
        ftime = fcst
        if (.not. fast) write(*,*) 'loop i,varname,glev=',i, varname, glev

!	suffix to name

!	_ave
        strlen = len_trim(varname)
	if (varname(strlen-3:strlen) == '_ave') then
            ftime=ave
            varname = varname(1:strlen-4)
        endif

!       some cases where ave is not in name
        if (fix_ave) then
           if (varname == 'snohf') ftime = ave
           if (varname == 'csulf') ftime = ave
           if (varname == 'csdlf') ftime = ave
           if (varname == 'csusf') ftime = ave
           if (varname == 'csdsf') ftime = ave
        endif

!	min_min
        strlen = len_trim(varname)
	if (varname(strlen-6:strlen) == 'min_min') then
            ftime=min
            varname = varname(1:strlen-7)
	    if (varname == 't') varname='tmp'
        endif

!	max_max
        strlen = len_trim(varname)
	if (varname(strlen-6:strlen) == 'max_max') then
            ftime=max
            varname = varname(1:strlen-7)
	    if (varname == 't') varname='tmp'
        endif

!	_acc
        strlen = len_trim(varname)
	if (varname(strlen-3:strlen) == '_acc') then
            ftime=acc
            varname = varname(1:strlen-4)
        endif

        strlen = len_trim(varname)
	if (varname(strlen-7:strlen) == '_hyblev1') then
            glev = '1 hybrid pressure level'
            varname = varname(1:strlen-8)
        endif

!	many of the nemsio levels use grib1 names
!	convert to grib1 levels to grib2

!       if (land_var) mask out sea and sea_ice region
        land_var = .false.

!	do the levels that are not grib2 compatible
	if (glev == 'sfc') glev='surface'
	if (glev == '0-10 cm down') then
!           0-0.1 m below ground (soil)
            glev = '0-0.1 m below ground'
	    if (convert_tsoil .and. varname == 'tmp') varname = 'TSOIL'
            land_var = .true.
        endif
	if (glev == '10-40 cm down') then
            glev='0.1-0.4 m below ground'
	    if (convert_tsoil .and. varname == 'tmp') varname = 'TSOIL'
            land_var = .true.
        endif
	if (glev == '40-100 cm down') then
            glev='0.4-1 m below ground'
	    if (convert_tsoil .and. varname == 'tmp') varname = 'TSOIL'
            land_var = .true.
        endif
        if (glev == '100-200 cm down') then
            glev='1-2 m below ground'
	    if (convert_tsoil .and. varname == 'tmp') varname = 'TSOIL'
            land_var = .true.
        endif
        if (glev == '0-200 cm down') then
            glev='0-2 m below ground'
	    if (convert_tsoil .and. varname == 'tmp') varname = 'TSOIL'
            land_var = .true.
        endif
	if (glev == 'nom. top') glev='top of atmosphere'
	if (glev == '2 m above gnd') glev='2 m above ground'
	if (glev == '10 m above gnd') glev='10 m above ground'
	if (glev == 'bndary-layer cld') glev='boundary layer cloud layer'
	if (glev == 'convect-cld laye') glev='convective cloud layer'
	if (glev == 'low cld lay') glev='low cloud layer'
	if (glev == 'mid cld lay') glev='middle cloud layer'
	if (glev == 'high cld lay') glev='high cloud layer'
	if (glev == 'low cld top') glev='low cloud top level'
	if (glev == 'mid cld top') glev='middle cloud top level'
	if (glev == 'high cld top') glev='high cloud top level'
	if (glev == 'low cld bot') glev='low cloud bottom level'
	if (glev == 'mid cld bot') glev='middle cloud bottom level'
	if (glev == 'high cld bot') glev='high cloud bottom level'
	if (glev == 'convect-cld top') glev='convective cloud top level'
	if (glev == 'convect-cld bot') glev='convective cloud bottom level'

	if (glev == 'mid layer') then
            glev = trim(itoa(reclev(i)-1)) // '-' // trim(itoa(reclev(i))) // ' hybrid pressure layer'
        endif

!	convert varname to uppercase
	varname = to_upper(varname)

!	handle varnames that are not grib2 

	if (varname == 'OROG') then
            varname = 'HGT'
        endif
	if (varname == 'DELZ') then
	    varname = 'THICK'
	endif

!	not sure what TPRCP is.  Looks like a precip but numbers are too small
!       use PRATE instead of TPRCP
!	if (varname == 'TPRCP') varname = 'PRATE'

	if (varname == 'CLWMR') varname = 'CLLMR'
	if (varname == 'CLD_AMT') then
!	    convert to %
	    varname = 'CDCA'
            do j = 1, im*jm
	        if (data1(j) .lt. 0.999*undefined) then
                    data1(j) = 100.0 * data1(j)
		endif
            enddo
	endif

	if (varname == 'VTYPE') varname = 'VGTYP'
	if (varname == 'SLC1') then
	    varname = 'SOILL'
	    glev='0-0.1 m below ground'
            land_var = .true.
	endif
	if (varname == 'SLC2') then
	    varname = 'SOILL'
	    glev='0.1-0.4 m below ground'
            land_var = .true.
	endif
	if (varname == 'SLC3') then
	    varname = 'SOILL'
	    glev='0.4-1 m below ground'
            land_var = .true.
	endif
	if (varname == 'SLC4') then
	    varname = 'SOILL'
            glev='1-2 m below ground'
            land_var = .true.
	endif
	if (varname == 'VGTYP'.or. varname == 'VEG' .or. varname=='SOTYP' &
           .or. varname == 'WILT' .or. varname == 'GFLUX' .or. varname == 'SFCR' &
           .or. varname == 'CNWAT') land_var = .true.

!	add some fields that need to be masked out for oceans
	if (varname == 'TRANS') land_var = .true.	! transpiration
	if (varname == 'FLDCP') land_var = .true.	! field capacity
	if (varname == 'EVBS') land_var = .true.	! evap over bare soil
	if (varname == 'EVCW') land_var = .true.	! evap by canapoy
	if (varname == 'WATR') land_var = .true.	! water runoff
	if (varname == 'PEVPR') land_var = .true.	! potential evap
	if (varname == 'SLTYP') land_var = .true.	! slope type
	if (varname == 'SSRUN') land_var = .true.	! storm runoff

!	get rid of nemsio undefined values by converting to wgrib2 undefined value
!omp parallel do private(j)
	do j = 1, im*jm
	    if (data1(j) .gt. 0.999*undefined .and. data1(j).lt.1.001*undefined) &
		data1(j) = undefined_wgrib2
	enddo

	if (testmode.and.varname == 'DPRES') then
	    press1 = press1 + data1
	endif

!       precip
!        write(*,*) 'varname=',trim(varname)

!	factors for PRATE, CPRAT
	if (varname == 'PRATE' .and. prate_adjust) then
            if (.not. fast) write(*,*) ' working on precip adjustment for PRATE factor=',precip_factor
!omp parallel do private(j)
            do j = 1, im*jm
	        if (data1(j) .lt. 0.999*undefined) then
                    data1(j) = data1(j) * precip_factor
                endif
            enddo
            ftime = precip_ave
        endif
	if (varname == 'CPRAT' .and. prate_adjust) then
            if (.not.fast) write(*,*) ' working on precip adjustment for CPRAT factor=',precip_factor
!omp parallel do private(j)
            do j = 1, im*jm
	        if (data1(j) .lt. 0.999*undefined) then
                    data1(j) = data1(j) * precip_factor
                endif
            enddo
            ftime = precip_ave
        endif
        if (has_landmask) then
  	    if (varname == 'ICEC' .or. varname == 'ICETK') then
!omp parallel do private(j)
                do j = 1, im*jm
		    if (land(j) .gt. 0.99 .and. land(j).lt.1.01) data1(j) = undefined_wgrib2
                enddo
            endif

	    if (land_var) then
!omp parallel do private(j)
                do j = 1, im*jm
!                   sea ice
		    if (land(j) .gt. 1.99 .and. land(j).lt.2.01) data1(j) = undefined_wgrib2
!                   water
		    if (land(j) .gt. -0.01 .and. land(j).lt.0.01) data1(j) = undefined_wgrib2
                enddo
            endif
        endif

!	save U10
	if (varname == 'UGRD' .and. glev == '10 m above ground' .and.  extra_var) then
	    u10m = data1
	    u10m_avail = .true.
	endif

!       save surface PRES
	if (varname == 'PRES'.and. glev == 'surface') then
           allocate(press0(im*jm))
           press0 = data1
           has_press0 = 1
        endif

!	write to grib file

        meta='d=' // trim(refdate) // ':' // trim(varname) // ':' // trim(glev) // ':' // trim(ftime) // ':' &
          // 'encode 16 bits:'
!	write(*,*) trim(meta)

        if (varname /= 'DPRES' .and. varname /= 'PRATEB' .and. varname /= 'CPRATB' .and. &
           varname /= 'TI' .and. varname /= 'TG3' .and.  varname /= 'F10M' .and. &
           varname /= 'FFHH' .and. varname /= 'FFMM' .and. varname /= 'F10') then
	    iret = grb2_wrt(trim(cgrb),'@mem:0',1,meta=meta,data1=data1, &
               order='we:ns', packing=packing)
	    if (iret /= 0 .and. .not. fast) then
	        write(*,*) 'bad meta: ', trim(meta)
	    endif
        endif

!	write WIND at 10m, assume U and V are undefined at same locations
	if (varname == 'VGRD' .and. glev == '10 m above ground' .and. u10m_avail .and. extra_var) then
!omp parallel do private(j)
            do j = 1, im*jm
	        if (data1(j) .lt. 0.999*undefined) then
                    u10m(j) = sqrt(u10m(j)*u10m(j) + data1(j)*data1(j))
                else
		    u10m(j) = undefined_wgrib2
		endif
            enddo
            meta='d=' // trim(refdate) // ':WIND:10 m above ground:' // trim(ftime) // ':encode 16 bits:'
	    iret = grb2_wrt(trim(cgrb),'@mem:0',1,meta=meta,data1=u10m, &
                order='we:ns', packing=packing)
	    if (iret /= 0 .and. .not. fast) then
	        write(*,*) 'bad meta: ', trim(meta)
	    endif
	    u10m_avail =  .false.
	endif
    enddo
!
   call nemsio_close(gfile,iret=iret)
!
   call nemsio_finalize()

   if (.not. fast) write(*,*) 'status of press calc'
   if (.not. fast) write(*,*) 'pres0,pres=',has_press0, has_press
   if (.not. fast) write(*,*) 'has_landmask=',has_landmask

   if (testmode) then
!     add 20.0 (pascal) for top of 64-level model
      data1 = press1 + 20.0
      meta='d=' // trim(refdate) // ':PRES:' // '0 m above ground:' // trim(ftime) // ':encode 16 bits:'
	 iret = grb2_wrt(trim(cgrb),'@mem:0',1,meta=meta,data1=data1, &
                order='we:ns', packing=packing)
	write(*,*) 'static surface pressure write err=',iret
   endif


!  write out pres for the model levels, assume 64 == pressure hybrid
!   if (has_press0 == 1 .and. lm == 64) then
!      do i = 1, lm
! !omp parallel do private(j)
!	  do j = 1, im*jm
!             data1(j) = p_hybrid64(i,press0(j))
!         enddo
!
!         meta='d=' // trim(refdate) // ':PRES:' // itoa(i) // ' hybrid pressure level:' // &
!                trim(ftime) // ':encode 16 bits:'
!         iret = grb2_wrt(trim(cgrb),'@mem:0',1,meta=meta,data1=data1, &
!                order='we:ns', packing=packing)
!     enddo
!   endif

!  finished with grib stuff, now to making the ctl file
   if (fast) stop 0

!
!---------------------------------------------------------------------------
!****** write .ctl file
!---------------------------------------------------------------------------
!
!-- get date
   call cmonth(idate(2),cmon)
   write(ctldate,'(i2.2,a,i2.2,a3,i4.4)')idate(4),'Z',idate(3)       &
        ,cmon,idate(1)

   io_unit=650
   open(io_unit,file=trim(cindx),form='formatted')

   write(io_unit,105)trim(cin)
   write(io_unit,106)
   write(io_unit,107)
   write(io_unit,108)tlmeta
   write(io_unit,109)
   write(io_unit,111)im,dxctl
   write(io_unit,112)jm
   write(io_unit,113)(lat(i),i=jm,1,-1)
   write(io_unit,114)
   if(nfhour/=0) then
     write(io_unit,115)1,trim(ctldate),nfhour
   else
     write(io_unit,116)1,trim(ctldate)
   endif

!
 105  FORMAT('dset ^',A)
 106  FORMAT('undef 9.99E+20')
 107  FORMAT('options big_endian sequential yrev')
 108  FORMAT('fileheader',I12.0)
 109  FORMAT('title gfs nemsioi sfc file')

 111  FORMAT('xdef  ',I6,' linear  0.000000 ',f9.6)
 112  FORMAT('ydef  ',I6,' levels')
 113  FORMAT(10f11.6)

 114  FORMAT('zdef 1 linear 1 1 ')
 115  FORMAT('tdef ',I6,' linear ',A12,' ',I6,'hr')
 116  FORMAT('tdef ',I6,' linear ',A12,'  1yr')
!
   krec=0
   do n=1,nrec
     if(reclev(n)==1) then
       krec=krec+1
     endif
   enddo
      
   WRITE(IO_UNIT,'(A,I6)')'VARS ',krec

   n=1
   do while (n<=nrec)
     reclevtyp_sht=reclevtyp(n)
     if(trim(reclevtyp_sht) == "convect-cld bot") then
       reclevtyp_sht="cvb"
     elseif (trim(reclevtyp_sht) == "convect-cld top") then
       reclevtyp_sht="cvt"
     elseif (trim(reclevtyp_sht) == "high cld bot") then
       reclevtyp_sht="hcb"
     elseif (trim(reclevtyp_sht) == "high cld top") then
       reclevtyp_sht="hct"
     elseif (trim(reclevtyp_sht) == "mid cld bot") then
       reclevtyp_sht="mcb"
     elseif (trim(reclevtyp_sht) == "mid cld top") then
       reclevtyp_sht="mct"
     elseif (trim(reclevtyp_sht) == "low cld bot") then
       reclevtyp_sht="lcb"
     elseif (trim(reclevtyp_sht) == "low cld top") then
       reclevtyp_sht="lct"
     elseif (trim(reclevtyp_sht) == "convect-cld laye") then
       reclevtyp_sht="cvcl"
     elseif (trim(reclevtyp_sht) == "bndary-layer cld") then
       reclevtyp_sht="bdrlc"
     elseif (trim(reclevtyp_sht) == "atmos col") then
       reclevtyp_sht="acol"
     elseif (trim(reclevtyp_sht) == "high cld lay") then
       reclevtyp_sht="hcl"
     elseif (trim(reclevtyp_sht) == "mid cld lay") then
       reclevtyp_sht="mcl"
     elseif (trim(reclevtyp_sht) == "low cld lay") then
       reclevtyp_sht="lcl"
     elseif (trim(reclevtyp_sht) == "2 m above gnd") then
       reclevtyp_sht="2m"
     elseif (trim(reclevtyp_sht) == "10 m above gnd") then
       reclevtyp_sht="10m"
     endif
     varname=sweep_blanks(trim(recname(n))//trim(reclevtyp_sht))
     if(trim(reclevtyp(n))=='mid layer') then
       write(io_unit,'(a16,i3,a)')varname,lm,' 99 model layer'
       n=n+lm
     elseif(trim(reclevtyp(n))=='soil layer') then
        write(io_unit,'(a16,i3,a)')varname,nsoil,' 99 soil layer'
        n=n+nsoil
     elseif(trim(reclevtyp(n))=='2 m above gnd') then
       recname(n)=trim(recname(n))//'2m'
       write(io_unit,'(a16,a7,a)')varname,'  0 99 ',trim(reclevtyp(n))
       n=n+1
     elseif(trim(reclevtyp(n))=='10 m above gnd') then
       recname(n)=trim(recname(n))//'10m'
       write(io_unit,'(a16,a7,a)')varname,'  0 99 ',trim(reclevtyp(n))
       n=n+1
     else
       write(io_unit,'(a16,a7,a)')varname,'  0 99 ',trim(reclevtyp(n))
       n=n+1
     endif
   enddo

   write(io_unit,'(A8)')'endvars'
   close(io_unit)

!---------------------------------------------------------------------------
!****** clean up
!---------------------------------------------------------------------------
  deallocate(recname,reclevtyp,reclev,lat,slat,dx)
!---------------------------------------------------------------------------
!
! - - - - -- - -- - -- - -- - - -- - --  -- - -- - -- - - -- - - - -- - --
  stop

 end program
! - - - - -- - -- - -- - -- - - -- - --  -- - -- - -- - - -- - - - -- - --
      SUBROUTINE CMONTH(IMON,CMON)
!
!-----------------------------------------------------------------------
!***  Convert month
!-----------------------------------------------------------------------
!
      INTEGER,INTENT(IN) :: IMON
      CHARACTER(LEN=3)   :: CMON
!
!-----------------------------------------------------------------------
!
      SELECT CASE (IMON)
        CASE(1)
            CMON='Jan'
        CASE(2)
            CMON='Feb'
        CASE(3)
            CMON='Mar'
        CASE(4)
            CMON='Apr'
        CASE(5)
            CMON='May'
        CASE(6)
            CMON='Jun'
        CASE(7)
            CMON='Jul'
        CASE(8)
            CMON='Aug'
        CASE(9)
            CMON='Sep'
        CASE(10)
            CMON='Oct'
        CASE(11)
            CMON='Nov'
        CASE(12)
            CMON='Dec'
      END SELECT
!
!-----------------------------------------------------------------------
!
      END SUBROUTINE CMONTH

!
      SUBROUTINE splat4(IDRT,JMAX,ASLAT)
!
      implicit none
      integer,intent(in) :: idrt,jmax
      real(4),intent(out) :: ASLAT(JMAX)
      INTEGER,PARAMETER:: KD=SELECTED_REAL_KIND(15,45)
      REAL(KIND=KD):: PK(JMAX/2),PKM1(JMAX/2),PKM2(JMAX/2)
      REAL(KIND=KD):: ASLATD(JMAX/2),SP,SPMAX,EPS=10.d0*EPSILON(SP)
      integer,PARAMETER:: JZ=50
      REAL(8) BZ(JZ)
      DATA BZ        / 2.4048255577d0,  5.5200781103d0, &
       8.6537279129d0, 11.7915344391d0, 14.9309177086d0, 18.0710639679d0, &
      21.2116366299d0, 24.3524715308d0, 27.4934791320d0, 30.6346064684d0, &
      33.7758202136d0, 36.9170983537d0, 40.0584257646d0, 43.1997917132d0, &
      46.3411883717d0, 49.4826098974d0, 52.6240518411d0, 55.7655107550d0, &
      58.9069839261d0, 62.0484691902d0, 65.1899648002d0, 68.3314693299d0, &
      71.4729816036d0, 74.6145006437d0, 77.7560256304d0, 80.8975558711d0, &
      84.0390907769d0, 87.1806298436d0, 90.3221726372d0, 93.4637187819d0, &
      96.6052679510d0, 99.7468198587d0, 102.888374254d0, 106.029930916d0, &
      109.171489649d0, 112.313050280d0, 115.454612653d0, 118.596176630d0, &
      121.737742088d0, 124.879308913d0, 128.020877005d0, 131.162446275d0, &
      134.304016638d0, 137.445588020d0, 140.587160352d0, 143.728733573d0, &
      146.870307625d0, 150.011882457d0, 153.153458019d0, 156.295034268d0 /
      REAL(8):: DLT,D1=1.d0
      INTEGER(4):: JHE,JHO,J0=0
      real(8),PARAMETER :: PI=3.14159265358979d0,C=(1.d0-(2.d0/PI)**2)*0.25d0
      real(8) r
      integer jh,js,n,j
!C - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
!C  GAUSSIAN LATITUDES
      IF(IDRT.EQ.4) THEN
        JH=JMAX/2
        JHE=(JMAX+1)/2
        R=1.d0/SQRT((JMAX+0.5d0)**2+C)
        DO J=1,MIN(JH,JZ)
          ASLATD(J)=COS(BZ(J)*R)
        ENDDO
        DO J=JZ+1,JH
          ASLATD(J)=COS((BZ(JZ)+(J-JZ)*PI)*R)
        ENDDO
        SPMAX=1.d0
        DO WHILE(SPMAX.GT.EPS)
          SPMAX=0.d0
          DO J=1,JH
            PKM1(J)=1.d0
            PK(J)=ASLATD(J)
          ENDDO
          DO N=2,JMAX
            DO J=1,JH
              PKM2(J)=PKM1(J)
              PKM1(J)=PK(J)
              PK(J)=((2*N-1)*ASLATD(J)*PKM1(J)-(N-1)*PKM2(J))/N
            ENDDO
          ENDDO
          DO J=1,JH
            SP=PK(J)*(1.d0-ASLATD(J)**2)/(JMAX*(PKM1(J)-ASLATD(J)*PK(J)))
            ASLATD(J)=ASLATD(J)-SP
            SPMAX=MAX(SPMAX,ABS(SP))
          ENDDO
        ENDDO
!
        DO J=1,JH
          ASLAT(J)=ASLATD(J)
          ASLAT(JMAX+1-J)=-ASLAT(J)
        ENDDO
        IF(JHE.GT.JH) THEN
          ASLAT(JHE)=0.d0
        ENDIF
!C - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
!C  EQUALLY-SPACED LATITUDES INCLUDING POLES
      ELSEIF(IDRT.EQ.0) THEN
        JH=JMAX/2
        JHE=(JMAX+1)/2
        JHO=JHE-1
        DLT=PI/(JMAX-1)
        ASLAT(1)=1.d0
!omp parallel do private(j)
        DO J=2,JH
          ASLAT(J)=COS((J-1)*DLT)
        ENDDO
!
        DO J=1,JH
          ASLAT(JMAX+1-J)=-ASLAT(J)
        ENDDO
        IF(JHE.GT.JH) THEN
          ASLAT(JHE)=0.d0
        ENDIF
!C - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
!C  EQUALLY-SPACED LATITUDES EXCLUDING POLES
      ELSEIF(IDRT.EQ.256) THEN
        JH=JMAX/2
        JHE=(JMAX+1)/2
        JHO=JHE
        DLT=PI/JMAX
        ASLAT(1)=1.d0
        DO J=1,JH
          ASLAT(J)=COS((J-0.5)*DLT)
        ENDDO
!
        DO J=1,JH
          ASLAT(JMAX+1-J)=-ASLAT(J)
        ENDDO
        IF(JHE.GT.JH) THEN
          ASLAT(JHE)=0.d0
        ENDIF
      ENDIF
!C - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
     end subroutine splat4

 character(35) function sweep_blanks(in_str)
!
   implicit none
!
   character(*), intent(in) :: in_str
   character(35) :: out_str
   character :: ch
   integer :: j

   out_str = " "
   do j=1, len_trim(in_str)
     ! get j-th char
     ch = in_str(j:j)
     if (ch .eq. "-") then
       out_str = trim(out_str) // "_"
     else if (ch .ne. " ") then
       out_str = trim(out_str) // ch
     endif
     sweep_blanks = out_str
   end do
 end function sweep_blanks

character(100) Function to_upper (str)

!   ==============================
!   Changes a string to upper case
!   ==============================

    Implicit None
    Character(*), Intent(In) :: str
    Character(LEN(str))      :: string

    Integer :: ic, i

    Character(26), Parameter :: cap = 'ABCDEFGHIJKLMNOPQRSTUVWXYZ'
    Character(26), Parameter :: low = 'abcdefghijklmnopqrstuvwxyz'

!   Capitalize each letter if it is lowecase
    to_upper = str
    do i = 1, LEN_TRIM(str)
        ic = INDEX(low, to_upper(i:i))
        if (ic > 0) to_upper(i:i) = cap(ic:ic)
    end do
    
End Function to_upper

character(30) Function itoa(i)

!   ==============================
!   Changes an integer to a string
!   ==============================

    Implicit none
    integer :: i
    character (30) :: string
    write(string,'(i9)') i
    itoa = adjustl(string)
End Function itoa


